############################ PSQF 6272 Example 6a using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("prediction")) install.packages("prediction")
library(prediction) # To get predicted values like Stata does

if (!require("ordinal")) install.packages("ordinal")
library(ordinal) # To estimate MLMs for ordinal outcomes

if (!require("mclogit")) install.packages("mclogit")
library(mclogit) # To estimate MLMs for nominal outcomes

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6a            #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example6a/"
filename = "Example6a_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "grade10"
Example6a = read_excel(paste0(filesave,filename), sheet="grade10") 
# Convert to data frame to use in analysis
Example6a = as.data.frame(Example6a)

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example6a_R_Output.txt"))

# Labels for original variables in comments only
# districtID= "districtID: District ID number"
# studentID=  "studentID: Student ID number"
# schoolID=   "schoolID: School ID number"
# lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
# math=       "math: Math Test Score"

print("R Descriptive Statistics within Student-Level Data")
prop.table(table(x=Example6a$lunch, useNA="ifany"))
describe(x=Example6a[ , c("math")])

# Create new lunch variables for clarity
Example6a$lunch3 = Example6a$lunch   # 3-category version
Example6a$lunch2=NA                  # 2-category version
Example6a$lunch2[which(Example6a$lunch>0)]=1  
Example6a$lunch2[which(Example6a$lunch==0)]=0

# Filter to only cases complete on all variables to be used below (before cluster means)
Example6a = Example6a[complete.cases(Example6a[ , c("math","lunch")]),]

# Create cluster mean math using Jonathan's function
Example6a = addUnitMeans(data=Example6a, unitVariable="schoolID",
                        meanVariables=c("math"), newNames=c("CMmath"))

print("R Descriptive Statistics within School-Level Data")
schoolMeans = unique(Example6a[,c("schoolID","NperschoolID","CMmath")])
describe(x=schoolMeans[ , c("NperschoolID","CMmath")])

# Rescale and center cluster mean math to be per 10 points
Example6a$CMmath50 = (Example6a$CMmath-50)/10
# CMmath50 = "CMmath50: School Mean Math (0=50)
# Rescale and cluster-MC student math to be per 10 points
Example6a$WCmath = (Example6a$math - Example6a$CMmath)/10
# WCmath= "WCmath: Within-School Math (0=CM)

print("R Descriptive Statistics within Student-Level Data")
describe(x=Example6a[ , c("WCmath")])
var(Example6a$WCmath)


################################################################################
#####                 BEGIN BINARY MODELS FOR EXAMPLE 6a                   #####
################################################################################

# NOTE: I am using 7 points of integration to match STATA default 
# until it forces me to use 1 instead for random slopes (to become Laplace)

print("R Model1: Empty Means, Single-Level for Student Binary Lunch")
Model1 = glm(data=Example6a, family=binomial(link="logit"), formula=lunch2~1)
summary(Model1) # residual deviance = -2LL already
print("Convert logits to probability via inverse link")
Model1Prob=1/(1+exp(-1*coefficients(Model1))); Model1Prob 

txtComment(" ") # insert blank space in output


print("R Model 2: Empty Means, Random Intercept for Student Binary Lunch")
Model2 = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=7,
               lunch2~1+(1|schoolID))
print("Show -2LL with more precision, results, and ICC using 3.29=residual variance")
-2*logLik(Model2); summary(Model2); icc(Model2)
print("Convert logits to probability via inverse link")
Model2Prob=1/(1+exp(-1*fixef(Model2))); Model2Prob 

print("Compute LRT manually -- would not work any other way")
DevTest=-2*(logLik(Model1)-logLik(Model2))
Pvalue=pchisq((DevTest), df=1, lower.tail=FALSE)
print("Test Statistic and P-values for DF=1") 
DevTest; Pvalue

txtComment(" ") # insert blank space in output


print("R Model 3: Add Level-2 Fixed Slope of School Mean Math")
Model3 = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=7, 
               lunch2~1+CMmath50+(1|schoolID))
print("Show -2LL with more precision, results, and odds ratios")
-2*logLik(Model3); summary(Model3); exp(fixef(Model3))

print("Yhat in logits and probabilities for unit-specific values of predictor") 
Model3Logits = prediction(model=Model3, type="link",     re.form=NA, at=list(CMmath50=-1:1))
Model3Probs  = prediction(model=Model3, type="response", re.form=NA, at=list(CMmath50=-1:1))
summary(Model3Logits); summary(Model3Probs)

txtComment(" ") # insert blank space in output


print("R Model 4: Add Level-1 Fixed Slope of Cluster-MC Student Math")
Model4 = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=7, 
               lunch2~1+CMmath50+WCmath+(1|schoolID))
print("Show -2LL with more precision, results, and odds ratios")
-2*logLik(Model4); summary(Model4); exp(fixef(Model4))
Model4glht = summary(glht(model=Model4, linfct=rbind(
    "Math Contextual Slope and Odds Ratio"= c(0,1,-1))),test=adjusted("none"))
Model4glht; data.frame(OR=exp(Model4glht$test$coefficients))

txtComment(" ") # insert blank space in output


print("R Model 5: Add Random Slope of Cluster-MC Student Math")
print("Switched to Laplace estimation with 1 quadrature point")
Model5 = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=1, 
               lunch2~1+CMmath50+WCmath+(1+WCmath|schoolID))
print("Show -2LL with more precision, results, and odds ratios")
-2*logLik(Model5); summary(Model5); exp(fixef(Model5))

print("Re-estimate Model 4 with same laplace method for LRT")
Model4R = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=1, 
                lunch2~1+CMmath50+WCmath+(1|schoolID))
print("LRT for random slope"); anova(Model5, Model4R)

txtComment(" ") # insert blank space in output


print("R Model 6: Add Interactions of School Mean and Cluster-MC Student Math")
print("Switched to Laplace estimation with 1 quadrature point")
Model6 = glmer(data=Example6a, family=binomial(link="logit"), nAGQ=1, 
               lunch2~1+CMmath50+WCmath+CMmath50:WCmath+I(CMmath50^2)+(1+WCmath|schoolID))
print("Show -2LL with more precision, results, and odds ratios")
-2*logLik(Model6); summary(Model6); exp(fixef(Model6))
Model6glht = summary(glht(model=Model6, linfct=rbind(
  "Math Contextual Simple Main Effect" = c(0,1,-1,0, 0),
  "Math Contextual Interaction"        = c(0,0, 0,1,-1))),test=adjusted("none"))
Model6glht; data.frame(OR=exp(Model6glht$test$coefficients))

txtComment(" ") # insert blank space in output


################################################################################
#####                BEGIN ORDINAL MODELS FOR EXAMPLE 6a                   #####
################################################################################

# NOTE: I am using 7 points of integration to match STATA default 
# until it forces me to use 1 instead for random slopes (to become Laplace)
# nominal_test(model) can be used to test proportional odds assumption in
#   models without random effects (clm only)

print("R Model 7: Empty Means, Single-Level for Student Ordinal Lunch")
print("Using clm from the ordinal package")
Model7 = clm(data=Example6a, link="logit", formula=as.factor(lunch3)~1)
print("Show -2LL and results") 
-2*logLik(Model7); summary(Model7)
print("Convert logits to probability via inverse link")
Model7Prob=1/(1+exp(-1*coefficients(Model7))); Model7Prob 

txtComment(" ") # insert blank space in output


print("R Model 8: Empty Means, Random Intercept for Student Ordinal Lunch")
print("Using clmm from the ordinal package")
Model8 = clmm(data=Example6a, link="logit", nAGQ=7,
              formula=as.factor(lunch3)~1+(1|schoolID))
print("Show -2LL, results, and ICC using 3.29=residual variance") 
-2*logLik(Model8); summary(Model8); icc(Model8)
print("LRT for Random Intercept Variance"); anova(Model8, Model7)
print("Convert logits to probability via inverse link")
Model8Prob=1/(1+exp(-1*coefficients(Model8))); Model8Prob 

txtComment(" ") # insert blank space in output


print("R Model 9: Add CMmath and Fixed Slope of Cluster-MC Student Math")
Model9 = clmm(data=Example6a, link="logit", nAGQ=7,
              formula=as.factor(lunch3)~1+CMmath50+WCmath+(1|schoolID))
print("Show -2LL, results, and odds ratios") 
-2*logLik(Model9); summary(Model9); exp(coefficients(Model9))
# Below does not work and I do not know why -- I have tried every combo I can think of
#Model9glht = summary(glht(model=Model9, linfct=rbind(
#  "Math Contextual Slope and Odds Ratio"= c(0,0,1,-1))),test=adjusted("none"))
#Model9glht; data.frame(OR=exp(Model9glht$test$coefficients))

txtComment(" ") # insert blank space in output


print("R Model 10: Add Random Slope of Cluster-MC Student Math")
Model10 = clmm(data=Example6a, link="logit", nAGQ=1,
               formula=as.factor(lunch3)~1+CMmath50+WCmath+(1+WCmath|schoolID))
print("Show -2LL, results, and odds ratios") 
-2*logLik(Model10); summary(Model10); exp(coefficients(Model10))

print("Re-estimate Model 9 with same laplace method for LRT")
Model9R = clmm(data=Example6a, link="logit", nAGQ=1,
               formula=as.factor(lunch3)~1+CMmath50+WCmath+(1|schoolID))
print("LRT for Random Slope Variance"); anova(Model10, Model9R)

txtComment(" ") # insert blank space in output


################################################################################
#####                BEGIN NOMINAL MODELS FOR EXAMPLE 6a                   #####
################################################################################

# NOTE: The results from mblogit were WAY OFF relative to those of SAS, STATA, 
# Mplus, and it may be because it uses penalized quasi-likelihood instead of
# true ML (adapative quadrature or Laplace, as in SAS and STATA).
# So use with caution!!

print("R Model 11: Empty Means, Single-Level for Student Nominal Lunch")
print("Model predicts conditional probability for 1 instead of 0 and 2 instead of 0")
Model11 = mblogit(data=Example6a, estimator="ML", formula=as.factor(lunch3)~1)
print("Show results"); summary(Model11)
print("Convert logits to probability via inverse link")
Model11Prob=1/(1+exp(-1*coefficients(Model11))); Model11Prob 

txtComment(" ") # insert blank space in output


print("R Model 12: Empty Means, Random Intercept for Student Nominal Lunch")
print("Model gives conditional probability for 1 instead of 0 and 2 instead of 0")
Model12 = mblogit(data=Example6a, estimator="ML", method="PQL",
                  formula=as.factor(lunch3)~1, random=~1|schoolID)
print("Show results"); summary(Model12)
print("Convert logits to probability via inverse link")
Model12Prob=1/(1+exp(-1*coefficients(Model12))); Model12Prob 

txtComment(" ") # insert blank space in output


print("R Model 13: Add CMmath and Fixed Slope of Cluster-MC Student Math")
print("Model gives conditional probability for 1 instead of 0 and 2 instead of 0")
Model13 = mblogit(data=Example6a, estimator="ML", method="PQL",
                  formula=as.factor(lunch3)~1+CMmath50+WCmath, random=~1|schoolID)
print("Show results and odds ratios") 
summary(Model13); exp(coefficients(Model13))
# Below does not work and I do not know why (I have tried every combo I can think of)
#Model13glht = summary(glht(model=Model13, linfct=rbind(
#  "Math Contextual Slope and Odds Ratio"= c(0,0,1,-1))),test=adjusted("none"))
#Model13glht; data.frame(OR=exp(Model13glht$test$coefficients))

txtComment(" ") # insert blank space in output


print("R Model 14: Add Random Slope of Cluster-MC Student Math")
print("Model gives conditional probability for 1 instead of 0 and 2 instead of 0")
print("Cannot use an LRT for random slopes given PQL estimation")
Model14 = mblogit(data=Example6a, estimator="ML", method="PQL",
                  formula=as.factor(lunch3)~1+CMmath50+WCmath, random=~1+WCmath|schoolID)
print("Show results and odds ratios") 
summary(Model14); exp(coefficients(Model14))

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
txtStop()


