// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6a         ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example6a"

// Open trimmed example excel data file from sheet "grade10" and clear away existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example6a_Data.xlsx", firstrow case(preserve) sheet("grade10") clear 
   
// Open external file to save results to
   log using $filesave\Example6a_STATA_Output.log, replace   

// Add labels to original variables
   label variable districtID "districtID: District ID number"
   label variable studentID  "studentID: Student ID number"
   label variable schoolID   "schoolID: School ID number"
   label variable lunch      "lunch: 0=Paid, 1=Reduced, 2=Free"
   label variable math       "math: Math Test Score"
     
display "STATA Descriptive Statistics within Student-Level Data"
tabulate lunch
summarize math      
     
// Create new lunch variables for clarity
   gen lunch3 = lunch // 3-category version
   gen lunch2 = .     // 2-category version
   replace lunch2=1 if lunch>0
   replace lunch2=0 if lunch==0
     
// Filter to complete cases before computing cluster means
   egen nmiss=rowmiss(math lunch)
   drop if nmiss>0

// Create sample size per school cluster mean math
   sort schoolID
   egen schoolN = count(math), by(schoolID)
   egen CMmath  = mean(math),  by(schoolID)  
   label variable schoolN "schoolN: # Students Sampled Per School" 
   label variable CMmath  "CMmath: School Mean Math Score"

// Rescale and center cluster mean math to be per 10 points
   gen CMmath50 = (CMmath-50)/10
   label variable CMmath50 "CMmath: School Mean Math (0=50)"

// Rescale and cluster-MC student math to be per 10 points
   gen WCmath = (math-CMmath)/10
   label variable WCmath "WCmath: Within-School Math (0=CM)"

display "STATA Descriptive Statistics within School-Level Data"
preserve  // Save for later use, then compute school-level dataset
collapse  schoolN CMmath, by(schoolID)
summarize schoolN CMmath
restore   // Go back to student-level dataset

display "STATA Descriptive Statistics within Student-Level Data"
summarize WCmath, detail


////////////////////////////////////////////////////////////////////////////////
////////                BEGIN BINARY MODELS FOR EXAMPLE 6a               ///////
////////////////////////////////////////////////////////////////////////////////

// Add option "or" to model options in melogit get odds ratios for fixed effects
// Using intpoints=7 as default

display "STATA Model 1: Empty Means, Single-Level for Student Binary Lunch"
melogit lunch2 , nolog 
display "-2LL = " e(ll)*-2      // Print -2LL for model  
nlcom 1/(1+exp(-1*(_b[_cons]))) // Fixed intercept in probability

display "STATA Model 2: Empty Means, Two-Level Logistic Model Predicting Lunch"
melogit lunch2 , ||  schoolID:  , intpoints(7) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // ICC using 3.29 as residual variance
nlcom 1/(1+exp(-1*(_b[_cons]))) // Fixed intercept in probability
     
display "STATA Model 3: Add Level-2 Fixed Slope of School Mean Math"
melogit lunch2 c.CMmath50, || schoolID: , intpoints(7) nolog 
display "-2LL = " e(ll)*-2                     // Print -2LL for model  
margins , at(c.CMmath50=(-1(1)1)) predict(xb)  // Unit-specific predicted logits (at U0c=0)
margins , at(c.CMmath50=(-1(1)1)) predict(mu)  // Marginal predicted probabilities
display "STATA Model 3: Odds Ratios Instead"
melogit lunch2 c.CMmath50, || schoolID: , intpoints(7) nolog or

display "STATA Model 4: Add Level-1 Fixed Slope of Cluster-MC Student Math"
melogit lunch2 c.CMmath50 c.WCmath, || schoolID: , intpoints(7) nolog
display "-2LL = " e(ll)*-2           // Print -2LL for model 
lincom c.WCmath*-1 + c.CMmath50*1    // Math Contextual Slope
estimates store FixMath              // Save LL for LRT
display "STATA Model 4: Odds Ratios Instead"
melogit lunch2 c.CMmath50 c.WCmath, || schoolID: , intpoints(7) nolog or
lincom c.WCmath*-1 + c.CMmath50*1, or  // Math Contextual Slope
    
display "STATA Model 5: Add Random Slope of Cluster-MC Student Math"
melogit lunch2 c.CMmath50 c.WCmath, || schoolID: WCmath, cov(un) intpoints(7) nolog // or
display "-2LL = " e(ll)*-2  // Print -2LL for model 
estimates store RandMath    // Save LL for LRT
lrtest RandMath FixMath     // LRT against fixed-only WCmath slope
display "STATA Model 5: Odds Ratios Instead"
melogit lunch2 c.CMmath50 c.WCmath, || schoolID: WCmath, cov(un) intpoints(7) nolog or
     
display "STATA Model 6: Add Interactions of School Mean and Cluster-MC Student Math"
melogit lunch2 c.CMmath50 c.WCmath c.CMmath50#c.WCmath c.CMmath50#c.CMmath50, ///
         || schoolID: WCmath, cov(un) intpoints(7) nolog // or
display "-2LL = " e(ll)*-2                               // Print -2LL for model 
lincom c.WCmath*-1 + c.CMmath50*1                        // Math Contextual Simple Main Effect
lincom c.CMmath50#c.WCmath*-1 + c.CMmath50#c.CMmath50*1  // Math Contextual Interaction
display "STATA Model 6: Odds Ratios Instead"
melogit lunch2 c.CMmath50 c.WCmath c.CMmath50#c.WCmath c.CMmath50#c.CMmath50, ///
         || schoolID: WCmath, cov(un) intpoints(7) nolog or
lincom c.WCmath*-1 + c.CMmath50*1, or                        // Math Contextual Simple Main Effect
lincom c.CMmath50#c.WCmath*-1 + c.CMmath50#c.CMmath50*1, or  // Math Contextual Interaction


////////////////////////////////////////////////////////////////////////////////
////////                BEGIN ORDINAL MODELS FOR EXAMPLE 6a              ///////
////////////////////////////////////////////////////////////////////////////////

// Using intpoints=7 as default

display "STATA Model 7: Empty Means, Single-Level for Student Ordinal Lunch"
meologit lunch3 , nolog  // coeflegend
display "-2LL = " e(ll)*-2       // Print -2LL for model  
nlcom 1/(1+exp(-1*(_b[/cut1])))  // 0 vs 12 threshold in probability
nlcom 1/(1+exp(-1*(_b[/cut2])))  // 01 vs 2 threshold in probability

display "STATA Model 8: Empty Means, Random Intercept for Student Ordinal Lunch"
meologit lunch3 , ||  schoolID:  , intpoints(7) nolog  // coeflegend
display "-2LL = " e(ll)*-2       // Print -2LL for model  
nlcom 1/(1+exp(-1*(_b[/cut1])))  // 0 vs 12 threshold in probability
nlcom 1/(1+exp(-1*(_b[/cut2])))  // 01 vs 2 threshold in probability
     
display "STATA Model 9: Add CMmath and Fixed Slope of Cluster-MC Student Math"
meologit lunch3 c.CMmath50 c.WCmath, || schoolID: , intpoints(7) nolog 
display "-2LL = " e(ll)*-2           // Print -2LL for model 
lincom c.WCmath*-1 + c.CMmath50*1    // Math Contextual Slope
estimates store FixMath              // Save LL for LRT
display "STATA Model 9: Odds Ratios Instead"
meologit lunch3 c.CMmath50 c.WCmath, || schoolID: , intpoints(7) nolog or
lincom c.WCmath*-1 + c.CMmath50*1, or    // Math Contextual Slope
    
display "STATA Model 10: Add Random Slope of Cluster-MC Student Math"
meologit lunch3 c.CMmath50 c.WCmath, || schoolID: WCmath, cov(un) intpoints(7) nolog
display "-2LL = " e(ll)*-2  // Print -2LL for model 
estimates store RandMath    // Save LL for LRT
lrtest RandMath FixMath     // LRT against fixed-only WCmath slope
display "STATA Model 10: Odds Ratios Instead"
meologit lunch3 c.CMmath50 c.WCmath, || schoolID: WCmath, cov(un) intpoints(7) nolog or


////////////////////////////////////////////////////////////////////////////////
////////                BEGIN NOMINAL MODELS FOR EXAMPLE 6a              ///////
////////////////////////////////////////////////////////////////////////////////

// Using intpoints=7 as default

display "STATA Model 11: Empty Means, Single-Level for Student Nominal Lunch"
display "Model predicts conditional probability for 1 instead of 0 and 2 instead of 0"
mlogit lunch3, nolog  // coeflegend
display "-2LL = " e(ll)*-2  // Print -2LL for model   
nlcom 1/(1+exp(-1*(_b[1:_cons])))  // 0 vs 1 conditional intercept in probability
nlcom 1/(1+exp(-1*(_b[2:_cons])))  // 0 vs 2 conditional intercept in probability

display "STATA Model 12: Empty Means, Random Intercept for Student Nominal Lunch"
display "Must be estimated as generalized SEM instead, 0 is ref category"
gsem (1.lunch3 <- M1[schoolID]@1)  ///
     (2.lunch3 <- M2[schoolID]@1), ///
     cov(M1[schoolID]*M2[schoolID]) latent(M1 M2) mlogit intpoints(7) nolog // coeflegend
display "-2LL = " e(ll)*-2  // Print -2LL for model   
nlcom 1/(1+exp(-1*(_b[1.lunch3:_cons])))  // 0 vs 1 conditional intercept in probability
nlcom 1/(1+exp(-1*(_b[2.lunch3:_cons])))  // 0 vs 2 conditional intercept in probability
     
display "STATA Model 13: Add Level-1 Fixed Slope of Cluster-MC Student Math"
display "nocapslatent over-rides default that capital letters = latent variables"
gsem (1.lunch3 <- c.CMmath50 c.WCmath M1[schoolID]@1)  ///
     (2.lunch3 <- c.CMmath50 c.WCmath M2[schoolID]@1), ///
     cov(M1[schoolID]*M2[schoolID]) nocapslatent latent(M1 M2) mlogit intpoints(7) nolog // coeflegend
display "-2LL = " e(ll)*-2          // Print -2LL for model 
lincom  _b[1.lunch3:WCmath]*-1 + _b[1.lunch3:CMmath50]*1  // Math Contextual Slope for 1 vs 0
lincom  _b[2.lunch3:WCmath]*-1 + _b[2.lunch3:CMmath50]*1  // Math Contextual Slope for 1 vs 2
estimates store FixMath             // Save LL for LRT
estat eform // Get odds ratios for solution
lincom  _b[1.lunch3:WCmath]*-1 + _b[1.lunch3:CMmath50]*1, or  // Math Contextual Slope for 1 vs 0
lincom  _b[2.lunch3:WCmath]*-1 + _b[2.lunch3:CMmath50]*1, or  // Math Contextual Slope for 1 vs 2
    
*display "STATA Model 14a: Add Random Slope of Cluster-MC Student Math"
*display "nocapslatent over-rides default that capital letters = latent variables"
*display "Will not converge using 7 quadrature points"
*gsem (1.lunch3 <- c.CMmath50 c.WCmath M1[schoolID]@1 c.WCmath#M1S[schoolID])  ///
*     (2.lunch3 <- c.CMmath50 c.WCmath M2[schoolID]@1 c.WCmath#M2S[schoolID]), ///
*     cov(M1[schoolID]*M2[schoolID]  M1S[schoolID]*M2S[schoolID] ///
*     M1[schoolID]*M2S[schoolID] M1S[schoolID]*M2[schoolID]) ///
*     nocapslatent latent(10 M2 M1S M2S) mlogit intpoints(7) nolog
*display "-2LL = " e(ll)*-2  // Print -2LL for model 
*estimates store RandMath    // Save LL for LRT
*lrtest RandMath FixMath     // LRT against fixed-only WCmath slope

*display "STATA Model 14b: Add Random Slope of Cluster-MC Student Math"
*display "nocapslatent over-rides default that capital letters = latent variables"
*display "Will not converge using 1 quadrature point (Laplace)"
*gsem (1.lunch3 <- c.CMmath50 c.WCmath M1[schoolID]@1 c.WCmath#M1S[schoolID])  ///
*     (2.lunch3 <- c.CMmath50 c.WCmath M2[schoolID]@1 c.WCmath#M2S[schoolID]), ///
*     cov(M1[schoolID]*M2[schoolID]  M1S[schoolID]*M2S[schoolID] ///
*     M1[schoolID]*M2S[schoolID] M1[schoolID]*M2[schoolID]) ///
*     nocapslatent latent(M1 M2 M1S M2S) mlogit intmethod(mcaghermite) intpoints(1) nolog
*display "-2LL = " e(ll)*-2  // Print -2LL for model 
*estimates store RandMath    // Save LL for LRT
*lrtest RandMath FixMath     // LRT against fixed-only WCmath slope


// Close external file of saved results
   log close
